@@ -11,7 +11,8 @@ from .model import Identifier, Mark, HumanMadeObject, Type, \
 	Acquisition, HumanMadeFeature, VisualItem, Set, Birth, Death, \
 	PropositionalObject, Payment, Creation, Phase, Period, \
 	Production, Event, DigitalObject, TransferOfCustody, \
-	STR_TYPES, factory
+	Move, DigitalService, CRMEntity, \
+	STR_TYPES, factory, ExternalResource
 
 # Add classified_as initialization hack for all resources
 def post_init(self, **kw):
@@ -74,7 +75,9 @@ ext_classes = {
 	"DoiIdentifier": {"parent": Identifier, "id": "300417432", "label": "DOI Identifer"},
 	"VolumeNumber": {"parent": Identifier, "id": "300265632", "label": "Volume"},
 	"IssueNumber": {"parent": Identifier, "id": "300312349", "label": "Issue"},
-
+	"CallNumber": {"parent": Identifier, "id": "300311706", "label": "Call Number"},
+	"StockNumber": {"parent": Identifier, "id": "300412177", "label": "Stock Number"},
+	
 	"EmailAddress": {"parent": Identifier, "id":"300435686", "label": "Email Address"},
 	"StreetAddress": {"parent": Identifier, "id":"300435687", "label": "Street Address"},
 	"TelephoneNumber": {"parent": Identifier, "id": "300435688", "label": "Telephone Number"},
@@ -94,7 +97,7 @@ ext_classes = {
 
 	"_BriefText": {"parent": LinguisticObject, "id": "300418049", "label":"Brief Text"},
 	"Note": {"parent": LinguisticObject, "id":"300027200", "label": "Note", "metatype": "brief text"},
-	"Description": {"parent": LinguisticObject, "id":"300411780", "label": "Description", "metatype": "brief text"},
+	"Description": {"parent": LinguisticObject, "id":"300435416", "label": "Description", "metatype": "brief text"},
 	"Abstract": {"parent": LinguisticObject, "id":"300026032", "label": "Abstract", "metatype": "brief text"},
 	"Citation": {"parent": LinguisticObject, "id":"300311705", "label": "Citation", "metatype": "brief text"},
 
@@ -106,16 +109,18 @@ ext_classes = {
 	"ContextStatement": {"parent": LinguisticObject, "id": "300435428", "label": "Historical/Cultural Context", "metatype": "brief text"},
 	"SourceStatement": {"parent": LinguisticObject, "id": "300404764", "label": "Source Statement", "metatype": "brief text"},
 	"PropertyStatusStatement": {"parent": LinguisticObject, "id": "300438433", "label": "Property Status Statement", "metatype": "brief text"},
+	"AccessStatement": {"parent": LinguisticObject, "id": "300133046", "label": "Access Statement", "metatype": "brief text"},
 
 	"MarkingsStatement": {"parent": LinguisticObject, "id": "300435420", "label": "Markings", "metatype": "brief text"},
 	"InscriptionStatement": {"parent": LinguisticObject, "id": "300435414", "label": "Inscription", "metatype": "brief text"},
-	"SignatureStatement": {"parent": LinguisticObject, "id": "300435415", "label": "Signature", "metatype": "brief text"},
+	"SignatureStatement": {"parent": LinguisticObject, "id": "300028705", "label": "Signature", "metatype": "brief text"},
 	"WatermarksStatement": {"parent": LinguisticObject, "id": "300435421", "label": "Watermarks", "metatype": "brief text"},
 	"MaterialStatement": {"parent": LinguisticObject, "id": "300435429", "label": "Material Statement", "metatype": "brief text"},
 	"PhysicalStatement": {"parent": LinguisticObject, "id": "300435452", "label": "Physical Statement", "metatype": "brief text"},
 	"DimensionStatement": {"parent": LinguisticObject, "id": "300435430", "label": "Dimension Statement", "metatype": "brief text"},
 	"CultureStatement": {"parent": LinguisticObject, "id": "300435431", "label": "Culture Statement", "metatype": "brief text"},
 	"PeriodStatement": {"parent": LinguisticObject, "id": "300435432", "label": "Period Statement", "metatype": "brief text"},
+	"EnvironmentStatement": {"parent": LinguisticObject, "id": "300229535", "label": "Environmental Conditions Statement", "metatype": "brief text"},
 
 	"ProductionStatement": {"parent": LinguisticObject, "id": "300435436", "label": "Production Statement", "metatype": "brief text" },
 	"AcquisitionStatement": {"parent": LinguisticObject, "id": "300435439", "label": "Acquisition Statement", "metatype": "brief text"},
@@ -123,6 +128,7 @@ ext_classes = {
 	"BibliographyStatement": {"parent": LinguisticObject, "id": "300026497", "label": "Bibliography Statement", "metatype": "brief text"},
 	"ExhibitionStatement": {"parent": LinguisticObject, "id":"300435424", "label":"Exhibition Statement", "metatype": "brief text"},
 	"PriceStatement": {"parent": LinguisticObject, "id":"300055694", "label":"Price Statement", "metatype": "brief text"},
+	"ConditionReport": {"parent": LinguisticObject, "id":"300379544", "label":"Condition Statement", "metatype": "brief text"},	
 
 	"BiographyStatement": {"parent": LinguisticObject, "id":"300435422", "label": "Biography Statement", "metatype": "brief text"},
 	"EditionStatement": {"parent": LinguisticObject, "id":"300435435", "label": "Edition Statement", "metatype": "brief text"},
@@ -170,6 +176,8 @@ ext_classes = {
 	"Diameter":   {"parent": Dimension, "id":"300055624", "label": "Diameter"},
 	"Weight":     {"parent": Dimension, "id":"300056240", "label": "Weight"},
 	"Color":      {"parent": Dimension, "id":"300080438", "label": "Color"}, # individual colors as dimensions, 56130 is concept of color
+	"Length":     {"parent": Dimension, "id":"300055645", "label": "Length"},
+	"Thickness":  {"parent": Dimension, "id":"300055646", "label": "Thickness"},
 	"SequencePosition": {"parent": Dimension, "id":"300010269", "label":"Sequence Position"},
 	"PhysicalDimension": {"parent": Dimension, "id":"300055642", "label":"Unknown physical dimension"},
 
@@ -188,6 +196,7 @@ ext_classes = {
 
 	"AuctionHouseOrg": {"parent": Group, "id": "300417515", "label": "Auction House (organization)"},
 	"MuseumOrg":   {"parent": Group, "id":"300312281", "label": "Museum"},
+	"Institution": {"parent": Group, "id":"300026004", "label": "Institution"},
 	"Department":  {"parent": Group, "id":"300263534", "label": "Department"},
 	"School":      {"parent": Group, "id":"300404284", "label": "School"},
 	"Studio":      {"parent": Group, "id":"300404275", "label": "Studio"},
@@ -204,6 +213,9 @@ ext_classes = {
 	"Style": 	   {"parent": Type, "id":"300015646", "label": "Style"},
 
 	"Auctioneer":  {"parent": Person, "id":"300025208", "label": "Auctioneer"}, # is this useful?
+	"Artist":  {"parent": Person, "id":"300025103", "label": "Artist"},
+	"Collector":  {"parent": Person, "id":"300025234", "label": "Collector"},
+	"Dealer":  {"parent": Person, "id":"300025236", "label": "Dealer"},
 
 	"AuctionEvent": {"parent": Activity, "id":"300054751", "label": "Auction Event"},
 	"Auction":     {"parent": Activity, "id":"300420001", "label": "Auction of Lot"}, # Individual auction-of-lot
@@ -234,8 +246,10 @@ ext_classes = {
 	"Managing": {"parent": Activity, "id":"300054277", "label": "Managing"},
 	"Storing": {"parent": Activity, "id":"300056390", "label": "Storing"},
 	"Producing": {"parent": Activity, "id":"300054713", "label": "Producing"},
+	"Conserving": {"parent": Activity, "id":"300404519", "label": "Conservation Activity"},
 
-	"ProvenanceEntry": {"parent": Activity, "id":"300055863", "label": "Provenance Entry"},
+	"ProvenanceEntry": {"parent": Activity, "id":"300055863", "label": "Provenance Activity"},
+	"ProvenanceActivity": {"parent": Activity, "id":"300055863", "label": "Provenance Activity"},
 	"ReturnOfLoan": {"parent": TransferOfCustody, "id":"300438467", "label": "Return of Loan"},
 	"Theft": {"parent": TransferOfCustody, "id": "300055292", "label": "Theft"},
 	"Looting": {"parent": TransferOfCustody, "id":"300379554", "label": "Looting"},
@@ -303,6 +317,7 @@ ext_classes = {
 	"Map": {"parent": HumanMadeObject, "id": "300028094", "label": "Map", "metatype": "work type"},
 	"Clothing": {"parent": HumanMadeObject, "id": "300266639", 'label': "Clothing", "metatype": "work type"},
 	"Furniture": {"parent": HumanMadeObject, "id":"300037680", "label": "Furniture", "metatype": "work type"},
+	"Sample": {"parent": HumanMadeObject, "id":"300028875", "label": "Sample", "metatype": "work type"},
 
 	"Architecture": {"parent": HumanMadeObject, "id":"300263552", "label": "Architecture", "metatype": "work type"},
 	"Armor": {"parent": HumanMadeObject, "id":"300036745", "label": "Armor", "metatype": "work type"},
@@ -331,6 +346,8 @@ ext_classes = {
 	"SalesCatalog": {"parent": HumanMadeObject,"id":"300026074", "label": "Sales Catalog", "metatype": "work type"},
 	"LotteryCatalog": {"parent": HumanMadeObject, "id":"300438603", "label": "Lottery Catalog", "metatype": "work type"},
 
+	"Sample": {"parent": HumanMadeObject, "id": "300028875", "label": "Sample"},
+
 	"FrontPart": {"parent": HumanMadeObject, "id":"300190703", "label": "Front Part", "metatype": "part type"},
 	"BackPart": {"parent": HumanMadeObject, "id":"300190692", "label": "Back Part", "metatype": "part type"},
 	"TopPart": {"parent": HumanMadeObject, "id":"300190710", "label": "Top Part", "metatype": "part type"},
@@ -372,17 +389,31 @@ identity_instances = {
 	"gelatin silver print": {"parent": Material, "id": "300128695", "label": "gelatin silver print"},
 	"silver": {"parent": Material, "id": "300011029", "label": "silver"},
 	"synthetic": {"parent": Type, "id": "xxx", "label": "Synthetic Material"},
+	"hazardous": {"parent": Material, "id": "300138222", "label": "Hazardous Material"},
+	"thread": {"parent": Material, "id": "300014250", "label": "Thread"},
 
-	# Measurement Units
+	# Measurement Units - lengths, weights, durations
 	"lignes": {"parent": MeasurementUnit, "id": "300435501", "label": "Paris lines"},
 	"fr_inches": {"parent": MeasurementUnit, "id": "300435502", "label": "Paris inches"},
 	"fr_feet": {"parent": MeasurementUnit, "id":"300435503", "label": "Paris feet"},
 	"inches": {"parent": MeasurementUnit, "id": "300379100", "label": "inches"},
 	"feet": {"parent": MeasurementUnit, "id":"300379101", "label": "feet"},
+	"mm": {"parent": MeasurementUnit, "id": "300379097", "label": "millimeters"},
 	"cm": {"parent": MeasurementUnit, "id": "300379098", "label": "centimeters"},
 	"meters": {"parent": MeasurementUnit, "id": "300379099", "label": "meters"},
 	"braccia": {"parent": MeasurementUnit, "id": "300404161", "label": "braccia"},
 	"ells": {"parent": MeasurementUnit, "id": "300412070", "label": "ells"},
+	"grams": {"parent": MeasurementUnit, "id": "300379225", "label": "grams"},
+	"kilograms": {"parent": MeasurementUnit, "id": "300379226", "label": "kilograms"},
+	"ounces":{"parent": MeasurementUnit, "id": "300379229", "label": "ounces"},
+	"pounds": {"parent": MeasurementUnit, "id": "300379254", "label": "pounds"},
+	"seconds": {"parent": MeasurementUnit, "id": "300379239", "label": "seconds"},
+	"minutes": {"parent": MeasurementUnit, "id": "300379240", "label": "minutes"},
+	"hours": {"parent": MeasurementUnit, "id": "300379241", "label": "hours"},
+	"days": {"parent": MeasurementUnit, "id": "300379242", "label": "days"},
+	"months": {"parent": MeasurementUnit, "id": "300379245", "label": "months"},
+	"years": {"parent": MeasurementUnit, "id": "300379244", "label": "years"},
+
 	"percent": {"parent": MeasurementUnit, "id": "300417377", "label": "percent"},
 	"numbers": {"parent": MeasurementUnit, "id": "300055665", "label": "numbers"},
 	"bytes": {"parent": MeasurementUnit, "id": "300265869", "label": "bytes"},
@@ -390,14 +421,10 @@ identity_instances = {
 	"megabytes": {"parent": MeasurementUnit, "id": "300265873", "label": "megabytes"},
 	"gigabytes": {"parent": MeasurementUnit, "id": "300265874", "label": "gigabytes"},
 	"terabytes": {"parent": MeasurementUnit, "id": "300266477", "label": "terabytes"},
-	"partsUnit": {"parent": MeasurementUnit, "id": "300404159", "label": "parts"},
 	"pageCount": {"parent": MeasurementUnit, "id": "300194222", "label": "pages"},
 	"pixels": {"parent": MeasurementUnit, "id": "300266190", "label": "pixels"},
 	"rgb_colorspace": {"parent": MeasurementUnit, "id": "300266239", "label": "rgb"},
-	"seconds": {"parent": MeasurementUnit, "id": "300379239", "label": "seconds"},
-	"days": {"parent": MeasurementUnit, "id": "300379242", "label": "days"},
-	"months": {"parent": MeasurementUnit, "id": "300379245", "label": "months"},
-	"years": {"parent": MeasurementUnit, "id": "300379244", "label": "years"},
+	"partsUnit": {"parent": MeasurementUnit, "id": "300404159", "label": "parts"},
 
 	# Languages
 	"english": {"parent": Language, "id": "300388277", "label": "English"},
@@ -494,6 +521,9 @@ identity_instances = {
 	"painting": {"parent": Type, "id":"300054216", "label": "Painting"},
 	"spraypainting": {"parent": Type, "id":"300053816", "label": "Spraypainting"},
 
+  # Conditions
+	"crizzling": {"parent": Type, "id": "300218594", "label": "Crizzling"},
+
 	# Geographic
 	"city": {"parent": Type, "id": "300008389", "label": "City"},
 	"province": {"parent": Type, "id":"300000774", "label": "Province"},
@@ -550,6 +580,7 @@ local_instances = {
 	# Nationalities
 	"american nationality": {"parent": Nationality, "id": "300107956", "label": "American"},
 	"austrian nationality": {"parent": Nationality, "id": "300111153", "label": "Austrian"},
+	"australian nationality": {"parent": Nationality, "id": "300021861", "label": "Australian"},
 	"belgian nationality": {"parent": Nationality, "id": "300111156", "label": "Belgian"},
 	"british nationality": {"parent": Nationality, "id": "300111159", "label": "British"},	
 	"canadian nationality": {"parent": Nationality, "id": "300107962", "label": "Canadian"},
@@ -568,6 +599,7 @@ local_instances = {
 	"italian nationality": {"parent": Nationality, "id": "300111198", "label": "Italian"},
 	"japanese nationality": {"parent": Nationality, "id": "300018519", "label": "Japanese"},
 	"mexican nationality": {"parent": Nationality, "id": "300107963", "label": "Mexican"},
+	"new zealand nationality": {"parent": Nationality, "id": "300021959", "label": "New Zealander"},
 	"norwegian nationality": {"parent": Nationality, "id": "300111201", "label": "Norwegian"},
 	"polish nationality": {"parent": Nationality, "id": "300111204", "label": "Polish"},
 	"portuguese nationality": {"parent": Nationality, "id": "300111207", "label": "Portuguese"},
@@ -575,7 +607,12 @@ local_instances = {
 	"spanish nationality": {"parent": Nationality, "id": "300111215", "label": "Spanish"},
 	"swedish nationality": {"parent": Nationality, "id": "300111218", "label": "Swedish"},
 	"swiss nationality": {"parent": Nationality, "id": "300111221", "label": "Swiss"},
+	"venetian nationality": {"parent": Nationality, "id": "300379657", "label": "Venetian"}, # republic until 1797
 
+	# Occupations
+	"artist occupation":  {"parent": Occupation, "id":"300025103", "label": "Artist"},
+ 	"collector occupation":  {"parent": Occupation, "id":"300025234", "label": "Collector"},
+ 	"dealer occupation":  {"parent": Occupation, "id":"300025236", "label": "Dealer"},
 
 	# Nations
 	"belgium": {"parent": Nation, "id": "1000063", "vocab": "tgn", "label": "Belgium"},
@@ -737,7 +774,7 @@ def add_attribute_assignment_check():
 def add_linked_art_boundary_check():
 
 	boundary_classes = [x.__name__ for x in [Actor, HumanMadeObject, Person, Group, VisualItem, \
-		Place, Period, LinguisticObject, Phase, Set, Event]]
+		Place, Period, LinguisticObject, Phase, Set, Event, DigitalObject, DigitalService]]
 	data_embed_classes = [Name, Identifier, Dimension, TimeSpan, MonetaryAmount]
 	type_embed_classes = [Type, Currency, Language, Material, MeasurementUnit]
 	event_embed_classes = [Birth, Creation, Production, Formation, Payment, \
@@ -751,9 +788,13 @@ def add_linked_art_boundary_check():
 	# Activity, AttributeAssignment, InformationObject, TransferOfCustody, Move
 	# Propositional Object
 
+	ExternalResource._embed_override = None
+
 	def my_linked_art_boundary_check(self, top, rel, value):
 		# True = Embed ; False = Split
-		if isinstance(value, LinguisticObject) and hasattr(value, 'classified_as'):
+		if value._embed_override is not None:
+			return value._embed_override
+		elif isinstance(value, LinguisticObject) and hasattr(value, 'classified_as'):
 			for ca in value.classified_as:
 				if instances['brief text'] in getattr(ca, 'classified_as', []):
 					return True
@@ -781,3 +822,48 @@ def add_linked_art_boundary_check():
 	setattr(BaseResource, "_linked_art_boundary_okay", my_linked_art_boundary_check)
 	factory.linked_art_boundaries = True	
 
+def set_linked_art_uri_segments():
+	HumanMadeObject._uri_segment = "object"
+	Activity._uri_segment = "event"
+	Event._uri_segment = "event"
+	Period._uri_segment = "event"
+	Place._uri_segment = "place"
+	InformationObject._uri_segment = "info"
+	Group._uri_segment = "group"
+	# Actor._uri_segment = "actor"
+	Person._uri_segment = "person"
+	PhysicalObject._uri_segment = "object"
+	LinguisticObject._uri_segment = "text"
+	PropositionalObject._uri_segment = "concept"
+	DigitalObject._uri_segment = "digital"
+	DigitalService._uri_segment = "digital"
+	Type._uri_segment = "concept"	
+	Language._uri_segment = "concept"
+	MeasurementUnit._uri_segment = "concept"
+	Currency._uri_segment = "concept"
+	Material._uri_segment = "concept"
+	VisualItem._uri_segment = "visual"
+	ProvenanceEntry._uri_segment = "provenance"
+	Exhibition._uri_segment = "activity"
+	Set._uri_segment = "set"
+
+
+def add_helper_functions():
+	# Add filter functions to the right bits of the model
+
+	def get_names(self, filter=None):
+		return [x for x in self.identified_by if isinstance(x, Name) and (not filter or filter in x.classified_as)]
+
+	def get_identifiers(self, filter=None):
+		return [x for x in self.identified_by if isinstance(x, Identifier) and (not filter or filter in x.classified_as)]
+
+	def get_statements(self, filter=None):
+		return [x for x in self.referred_to_by if isinstance(x, LinguisticObject) and x.content and (not filter or filter in x.classified_as)]
+
+	CRMEntity.get_names = get_names
+	CRMEntity.get_identifiers = get_identifiers
+	CRMEntity.get_statements = get_statements
+
+
+
+
